#include <avr/io.h>
#include <avr/interrupt.h> /* for sei() */
#include <util/delay.h>    /* for _delay_ms() */
#include <avr/pgmspace.h>  /* required by usbdrv.h */

#include "money.h"
#include "usbdrv.h"

#define RQ_GET_PIEZO1 0
#define RQ_GET_PIEZO2 1
#define RQ_GET_PIEZO3 2
#define RQ_GET_PIEZO4 3
#define RQ_SET_LED 4

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint16_t piezo_value;

    if (rq->bRequest == RQ_GET_PIEZO1)
    {
        piezo_value = read_adc(PC0);
        usbMsgPtr = (uchar *)&piezo_value;
        return sizeof(piezo_value);
    }

    else if (rq->bRequest == RQ_GET_PIEZO2)
    {
        piezo_value = read_adc(PC1);
        usbMsgPtr = (uchar *)&piezo_value;
        return sizeof(piezo_value);
    }

    else if (rq->bRequest == RQ_GET_PIEZO3)
    {
        piezo_value = read_adc(PC3);
        usbMsgPtr = (uchar *)&piezo_value;
        return sizeof(piezo_value);
    }

    else if (rq->bRequest == RQ_GET_PIEZO4)
    {
        piezo_value = read_adc(PC4);
        usbMsgPtr = (uchar *)&piezo_value;
        return sizeof(piezo_value);
    }

    else if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t value = rq->wValue.bytes[0];
        set_led(value);
        return 0;
    }

    /* default for not implemented requests: return no data back to host */
    return 0;
}

/* ------------------------------------------------------------------------- */
int main(void)
{
    init_money();
    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for (;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */

